--beginvalidatingquery
IF EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = 'dbo' AND TABLE_NAME = 'tblFindDatabaseVersion')
    BEGIN
    declare @major int = 16, @minor int = 2, @patch int = 0
    IF EXISTS (SELECT 1 FROM dbo.tblFindDatabaseVersion WHERE Major = @major AND Minor = @minor AND Patch = @patch)
        SELECT 0,'Already correct database version'
    ELSE
        SELECT 1, 'Upgrading database'
    END
ELSE
    SELECT -1, 'Not an EPiServer database with Find'
--endvalidatingquery

GO


IF NOT EXISTS (SELECT 1 FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME = N'tblFindIndexingJobQueue' AND TABLE_SCHEMA = N'dbo')
BEGIN
    CREATE TABLE [dbo].[tblFindIndexingJobQueue](
        [Id] [bigint] IDENTITY(1,1) NOT NULL,
        [Action] [smallint] NOT NULL,
        [SiteId] [nvarchar](255) NULL,
        [Language] [nvarchar](32) NULL,
        [Data] [nvarchar](MAX) NULL,
        [TimeStamp] [datetimeoffset] NOT NULL,
        CONSTRAINT [PK_tblFindIndexingJobQueue] PRIMARY KEY CLUSTERED
            (
                [Id] ASC
            )WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
        ) ON [PRIMARY]
END
GO


-- DROP OLD OBJECTS AND TYPES FIRST
IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueCount]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueCount]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueLoadAll]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueLoadAll]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueLoadItems]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueLoadItems]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueDeleteItem]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueDeleteItem]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueDeleteItems]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueDeleteItems]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueDeleteAll]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueDeleteAll]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueSave]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueSave]
END
GO

IF EXISTS (SELECT * FROM dbo.sysobjects WHERE id = object_id (N'[dbo].[findIndexJobQueueSaveMany]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
BEGIN
    DROP PROCEDURE [dbo].[findIndexJobQueueSaveMany]
END
GO

IF EXISTS (SELECT 1 FROM sys.types WHERE is_table_type = 1 AND name ='findBigIDTable')
BEGIN
    DROP TYPE [dbo].[findBigIDTable]
END
GO

IF EXISTS (SELECT 1 FROM sys.types WHERE is_table_type = 1 AND name ='findIndexingJobQueueTable')
BEGIN
    DROP TYPE [dbo].[findIndexingJobQueueTable]
END
GO


-- CREATE NEW OBJECTS AND TYPES
CREATE TYPE [dbo].[findBigIDTable] AS TABLE(
    [ID] [bigint] NOT NULL
)
GO


CREATE TYPE [dbo].[findIndexingJobQueueTable] AS TABLE(
    [Action] [smallint] NOT NULL,
    [SiteId] [nvarchar](255) NULL,
    [Language] [nvarchar](32) NULL,
    [Data] [nvarchar](MAX) NULL,
    [TimeStamp] [datetimeoffset] NOT NULL
)
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueCount]
AS
    BEGIN
        SELECT COUNT_BIG(Id) FROM tblFindIndexingJobQueue
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueLoadAll]
AS
    BEGIN
        SET NOCOUNT ON
        SELECT [Id], [Action], SiteId, [Data], [Language], [TimeStamp] FROM tblFindIndexingJobQueue ORDER BY Id
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueLoadItems]
    (
        @items INT
    )
    AS
    BEGIN
        SET NOCOUNT ON
        SELECT TOP (@items) [Id], [Action], SiteId, [Data], [Language], [TimeStamp] FROM tblFindIndexingJobQueue ORDER BY Id
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueDeleteItem]
    (
        @id BIGINT
    )
    AS
    BEGIN
        SET NOCOUNT ON
        DELETE FROM tblFindIndexingJobQueue WHERE [Id] = @id
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueDeleteItems]
    (
        @ids findBigIDTable READONLY
    )
    AS
    BEGIN
        SET NOCOUNT ON
        DELETE FROM tblFindIndexingJobQueue WHERE [Id] IN(SELECT [ID] FROM @ids)
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueDeleteAll]
AS
    BEGIN
        SET NOCOUNT ON
        DELETE FROM tblFindIndexingJobQueue
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueSave]
    (
        @action int,
        @siteId nvarchar(255),
        @itemLanguage nvarchar(32),
        @data nvarchar(MAX),
        @timeStamp datetimeoffset
    )
    AS
    BEGIN
        SET NOCOUNT ON
        INSERT INTO tblFindIndexingJobQueue([Action], SiteId, [Data], [Language], [TimeStamp])
        VALUES(@action, @siteId, @data, @itemLanguage, @timeStamp)
    END
GO


CREATE PROCEDURE [dbo].[findIndexJobQueueSaveMany]
    (
        @jobQueueItems findIndexingJobQueueTable READONLY
    )
    AS
    BEGIN
        SET NOCOUNT ON
        INSERT INTO tblFindIndexingJobQueue([Action], SiteId, [Data], [Language], [TimeStamp])
            SELECT  [Action], SiteId, [Data], [Language], [TimeStamp] FROM @jobQueueItems
    END
GO



INSERT INTO tblFindDatabaseVersion(Major, Minor, Patch) VALUES (16,2,0)

GO

PRINT N'Update complete.';